@@ -48,7 +48,10 @@ app = new Subject([
 
 // we wait for the addon object before loading anything
 app.subscribe('addon', function() {
-  if (!app.loadState()) app.openDefaultTemplate();
+  if (!app.loadState()) {
+    app.openDefaultTemplate();
+    app.newConnector();
+  } 
 });
 app.set('proxied', false);
 app.set("id", APP_ID);
