@@ -1,9 +1,13 @@
 EXPORTED_SYMBOLS = ["taskPane"];
 const DEBUG = true;
+const HTMLNS = "http://www.w3.org/1999/xhtml";
+const NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
+Components.utils.import("resource://indexdata/runtime/core.js");
 Components.utils.import("resource://indexdata/ui/app.js");
 Components.utils.import("resource://indexdata/util/logging.js");
 Components.utils.import("resource://indexdata/ui/testEditor.js");
 Components.utils.import("resource://indexdata/ui/StepTree.js");
+Components.utils.import("resource://indexdata/ui/DataBrowser.js");
 Components.utils.import("resource://indexdata/runtime/ConnectorTemplate.js");
 Components.utils.import("resource://indexdata/runtime/Connector.js");
 Components.utils.import("resource://indexdata/runtime/Task.js");
@@ -13,18 +17,13 @@ Components.utils.import("resource://indexdata/util/xmlHelper.js");
 Components.utils.import("resource://indexdata/util/xulHelper.js");
 Components.utils.import("resource://indexdata/util/textHelper.js");
 Components.utils.import("resource://indexdata/util/io.js");
-
-const HTMLNS = "http://www.w3.org/1999/xhtml";
-const NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
-
-
-//save some typing
+// abbreviations
 var child = xmlHelper.appendNode;
 var empty = xmlHelper.emptyChildren;
-
-
-//redirect 'test' logging to runMeter, carefull since at times run meter
-//may not exist, also filter DEBUG messages
+var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
+.getService(Components.interfaces.nsIPromptService);
+// redirect 'test' logging to runMeter, careful since at times run meter
+// may not exist, also filter DEBUG messages
 logging.getLogger('test_execution').addAppender({
   append: function (prefix, level, msg) {
     if (level === 'DEBUG') return;
@@ -33,11 +32,10 @@ logging.getLogger('test_execution').addAppender({
   },
   toString: function() { return "RMAppender"; }
 });
-
 //retrieve the default logger
 var logger = logging.getLogger();
-
-app.subscribe("canSave", function (canSave) {
+var callbacks = {
+  canSave: function (canSave) {
     if (canSave) {
       logger.debug("taskPane: enabling save button");
       taskPane.enableSave();
@@ -45,33 +43,27 @@ app.subscribe("canSave", function (canSave) {
       logger.debug("taskPane: disabling save button");
       taskPane.disableSave();
     }
-});
-
-app.subscribe("lastOpened", function (lastOpened) {
+  },
+  lastOpened: function (lastOpened) {
     logger.debug("taskPane: lastOpened file name changed");
     // only applies if task pane was open, otherwise wait until onTaskPaneLoad
     if (app.isSidebarOn) taskPane.setSidebarTitle();
-});
-
-app.subscribe("testNameChanged", function () {
+  },
+  testNameChanged: function () {
     logger.debug("Test name changed notified");
     // only applies if task pane was open, otherwise wait until onTaskPaneLoad
     if (app.isSidebarOn) taskPane.refreshTaskTests();
-});
-
-app.subscribe("fullqueryEdited", function (editedValue) {
+  },
+  fullqueryEdited: function (editedValue) {
     logger.debug("Fullquery has been edited through the editor");
     testEditor.getActiveTest().setArg('fullquery', editedValue);
     taskPane.refreshArguments();
-});
-
-
-// this is THE ONLY callback that calls the method to refresh the pane
-
-// TODO this needs to have more granularity -
-// seperate events for task arg updates, step list and so on
-// for now just refresh the whole pane aytime anything changes
-app.subscribe("selectedTask", function (selectedTask) {
+  },
+  // this is THE ONLY callback that calls the method to refresh the pane
+  // TODO this needs to have more granularity -
+  // seperate events for task arg updates, step list and so on
+  // for now just refresh the whole pane aytime anything changes
+  selectedTask: function (selectedTask) {
     //what follows assumes the sidebar (taskPane) is loaaded!
     //DO NOT force 'toggleSidebar' in here, it's async and won't help
     //instead we might consider moving all "refresh" to a sidebar calback
@@ -87,18 +79,18 @@ app.subscribe("selectedTask", function (selectedTask) {
       app.selectedTask.addHandler("onTaskFinish",
         function (outcome, results, errpos) {
           if (!outcome) taskPane.highlightError(errpos);
-          taskPane.refreshData();
+          taskPane.dataBrowser.refresh();
           taskPane.refreshArguments();
         }
       );
     }
     if (app.isSidebarOn) {
       taskPane.stepTree.setTask(app.selectedTask);
+      taskPane.dataBrowser.setTask(app.selectedTask);
       taskPane.refreshTaskPane();
     }
-});
-
-app.subscribe("connector", function (connector) {
+  },
+  connector: function (connector) {
     logger.debug("taskPane: connector has changed, force refresh...");
     connector
       .setTaskArgumentsUpdateCb(taskPane.onTaskArgumentsUpdate);
@@ -108,22 +100,19 @@ app.subscribe("connector", function (connector) {
     var tasks = app.connector.findTasks(
       app.connector.template.properties["defaultTaskSelection"]);
     if (!tasks && app.isSidebarOn) {
+      app.popupError("Default task not found, you may have the wrong template. Current template is: " + app.template.name);
       logger.debug('Cannot find default task to select, try refresh with null');
       app.set("selectedTask", null);
     } else {
       logger.debug('Selected task changed to ' + tasks[0].name);
       app.set("selectedTask", tasks[0]);
     }
-});
-
-app.subscribe("template", function () {
+  },
+  template: function () {
     logger.debug("changed template: modifying sidebar title");
     taskPane.setSidebarTitle();
-});
-
-var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
-.getService(Components.interfaces.nsIPromptService);
-
+  }
+};
 // data structure for the builder interface.
 var taskPane = {
   // wait for the main window
@@ -136,35 +125,33 @@ var taskPane = {
   mainWindow: null,
   runMeter: null,
   selectedStep: null,
-  dataTab: 1,
   stepClip: [],
-
+  subscribe: function () {
+    for (let cb in callbacks) app.subscribe(cb, callbacks[cb]);
+  },
+  unsubscribe: function () {
+    for (let cb in callbacks) app.unsubscribe(cb, callbacks[cb]);
+  },
   setDisabledTestControls: function (disable) {
     taskPane.tpDoc.getElementById('cfTaskTests').disabled = disable;
     //taskPane.tpDoc.getElementById('cfTestAdd').disabled = disable;
     taskPane.tpDoc.getElementById('cfTestRemove').disabled = disable;
     taskPane.tpDoc.getElementById('cfTestEdit').disabled = disable;
   },
-
   onTaskArgumentsUpdate: function() {
     taskPane.refreshTaskMenu();
     taskPane.refreshArguments();
   },
-
   onStepParametersUpdate: function(step) {
     logger.debug("step parameters updated for " + step );
     app.set("canSave", true);
     taskPane.stepTree.refresh();
   },
-
   onNewTaskLoad: function (e) {
-    if (!app.template)
-      throw new Error("no template loaded");
-
+    if (!app.connector.template) throw new Error("no template loaded");
     var doc = e.target;
     var list = doc.getElementById("cfNewTaskList");
-    var taskTemplates = app.template.getTaskTemplates();
-
+    var taskTemplates = app.connector.template.getTaskTemplates();
     for (var i=0; i<taskTemplates.length; i++) {
       var tt = taskTemplates[i];
       var item = xmlHelper.appendNode(list, "listitem", null, {"label":tt.name, "value":tt.name})
@@ -174,23 +161,21 @@ var taskPane = {
           {"label":tt.getArgNames().join(", ")})
     }
   },
-
   addNewTask: function (taskName) {
     var task = app.connector.createTask(taskName)
     task.createDefaultTests();
     app.set("selectedTask", task);
     app.set("canSave", true);
   },
-
   onNewTaskUnload: function(e) {
     e.target.defaultView.removeEventListener('load',
         taskPane.onNewTaskLoad, false);
     e.target.defaultView.removeEventListener('unload',
         taskPane.onNewTaskUnload, false);
   },
-
   onTaskPaneLoad: function (e) {
     logger.debug("onTaskPaneLoad: window ready, preparing task pane...");
+    taskPane.subscribe();
     //not be fully safe (see next comment), should we check the originalTarget?
     app.set("isSidebarOn", true);
     //IMPORTANT! ------------------------
@@ -236,16 +221,17 @@ var taskPane = {
         return;
       }
     });
-
+    // set up the data browser
+    let dbel = taskPane.tpDoc.getElementById("cfDataBrowser")
+    let dbcontext = taskPane.tpDoc.getElementById("dataBrowserContextMenu");
+    taskPane.dataBrowser = new DataBrowser();
+    taskPane.dataBrowser.init(dbel, dbcontext);
 
     //check licence and disallow the task pane if not accepted.
     //if so disable the builder
     var isLicenceAccepted = function () {
-      var prefs = Components.classes["@mozilla.org/preferences-service;1"]
-        .getService(Components.interfaces.nsIPrefService)
-        .getBranch("indexdata.cf.");
       try {
-        if (prefs.getIntPref("licenceaccepted") > 0)
+        if (core.idprefs.getIntPref("licenceaccepted") > 0)
           return true;
         else
           return false;
@@ -275,15 +261,10 @@ var taskPane = {
       app.connector.onStepParametersUpdate(taskPane.selectedStep);
     }, false);
 
-    //if we deal with another firefox window, we work on the same connector
-    if (app.connector == null) {
-      if (!app.loadState())
-        app.openDefaultTemplate();
-    } else {
-      //THIS IS SOO HACKY
-      //otherwsie the task pane won't be redrawn if it was re-opened
-      app.notify("connector");
-    }
+    //THIS IS SOO HACKY
+    //otherwsie the task pane won't be redrawn if it was re-opened
+    if (app.connector) app.notify("connector");
+
     // The sidebar title needs setting, because it is cached from
     // the previous run and displays the name of a no-longer-loaded
     // connector if we leave it.  And if the task pane is closed and
@@ -291,9 +272,9 @@ var taskPane = {
     // the overlay.
     taskPane.setSidebarTitle();
   },
-
   onTaskPaneUnload: function (e) {
     logger.debug("onTaskPaneUnload: window destroyed");
+    taskPane.unsubscribe();
     taskPane.mainWindow = null;
     app.set("isSidebarOn", false);
     // This will be selected via Task.selectedStepIndex
@@ -307,7 +288,6 @@ var taskPane = {
     //also all unregistered listeners will get register back again in the
     //chrome/xul when sidebar is opened again
   },
-
   onTaskSelect: function (t) {
     app.selectedTask = app.connector.getTaskById(t);
     if (typeof (app.selectedTask.selectedStepIndex) === 'undefined') 
@@ -315,7 +295,6 @@ var taskPane = {
     taskPane.selectedStep = null;
     app.notify("selectedTask");
   },
-
   // populate args from test, check task findability, set up data refresh
   prepTask: function(task) {
     let test = testEditor.getActiveTest();
@@ -334,7 +313,6 @@ var taskPane = {
       }
     }
   },
-
   onRunAll: function() {
     let task = app.selectedTask;
     taskPane.bindToRunMeter();
@@ -347,11 +325,9 @@ var taskPane = {
     taskPane.prepTask(task);
     task.run();
   },
-
   highlightError: function (pos) {
     //TODO not implemented
   },
-
   bindToRunMeter: function () {
     taskPane.stepSplit.setAttribute("collapsed", "false");
     taskPane.stepBox.setAttribute("collapsed", "false");
@@ -361,7 +337,6 @@ var taskPane = {
     app.selectedTask.addHandler("onMessageLog", taskPane.updateMeterLog);
     app.selectedTask.addHandler("onTaskFinish", taskPane.updateMeterOutcome);
   },
-
   drawRunMeter: function (surface) {
     xmlHelper.emptyChildren(surface);
     var vbox = xmlHelper.appendNode(surface, "vbox",null,
@@ -378,7 +353,6 @@ var taskPane = {
         {"label":"Log message"});
     return vbox.ownerDocument;
   },
-
   onRunSelected: function () {
     let items = taskPane.stepTree.getValidSelection()[1];
     let from = items.length > 0 ? items[0].block.findItemIndex(items[0]) : 0;
@@ -404,11 +378,9 @@ var taskPane = {
       }
     }
   },
-
   popupError: function (errorMsg) {
     taskPane.mainWindow.alert(errorMsg);
   },
-
   removeCurrentTask: function () {
     if (app.selectedTask) {
       if (prompts.confirm(null, "Please confirm",
@@ -421,18 +393,15 @@ var taskPane = {
       prompts.alert(null, "Warning", "No selected task to delete.");
     }
   },
-
   openTaskWindow: function (mainWin) {
     mozHelper.openWindow(taskPane.onTaskPaneLoad,
         "chrome://cfbuilder/content/task_window.xul");
   },
-
   openStepWindow: function (mainWin) {
     mozHelper.openWindow(function (e) {
         taskPane.stepBox = e.target.getElementById("cfStepBox");
         } , "chrome://cfbuilder/content/step_window.xul");
   },
-
   openRunMeter: function (mainWin) {
     mozHelper.openWindow(function (e) {
         taskPane.runMeter = e.target;
@@ -479,28 +448,28 @@ var taskPane = {
     var pmBar = taskPane.runMeter.getElementById("cfRunProgress");
     pmBar.value = 0;
   },
-
   setSidebarTitle: function () {
     var string = "";
     if (app.lastOpened && app.lastOpened.path) {
       string += app.lastOpened.path.replace(/.*[\/\\]/, "")
         + " (local)";
     }
-    if (app.connector && app.connector.metaData.origin) {
+    if (app.connector && app.connector.metaData && app.connector.metaData.origin) {
       if (string) string += ", ";
       string += app.connector.metaData.origin.replace(/.*[\/\\]/, "")
         + " (repo)";
     }
     if (!string) string = "Unnamed Connector";
-    if (app.templateFile) {
-      string += " [" + app.templateFile.path.replace(/.*[\/\\]/, "") + "]";
+    if (app.connector && app.connector.template && app.connector.template.name) {
+      string += " [" + app.connector.template.name + "]";
     }
     var node = taskPane.mainWindow.document.getElementById("sidebar-title");
     node.value = string;
     node.setAttribute("style", "font-weight: bold");
     node.setAttribute("tooltiptext", string);
   },
-
+  
+  // Tests
   runCurrentTest: function () {
     if (testEditor.getActiveTest() == null)
       logger.debug("Current test is null");
@@ -509,7 +478,6 @@ var taskPane = {
       testEditor.getActiveTest().run();
     }
   },
-
   onTestSelect: function (idx) {
     if (app.selectedTask == null) return;
     logger.debug("Switching test to " + idx );
@@ -518,7 +486,6 @@ var taskPane = {
     //FIXME this should be done using publish/subscribe model (via app)
     taskPane.refreshArguments();
   },
-
   onTestRemove: function (idx) {
     logger.debug("Removing test " + idx);
     idx = parseInt(idx);
@@ -533,7 +500,6 @@ var taskPane = {
     taskPane.refreshTaskTests();
     taskPane.refreshArguments();
   },
-
   addTest: function (testName) {
     var test = app.selectedTask.createTest(testName);
     taskPane.switchActiveTest(test);
@@ -541,12 +507,10 @@ var taskPane = {
     taskPane.refreshTaskTests();
     taskPane.setDisabledTestControls(false);
   },
-
   switchActiveTest: function (test) {
     logger.debug("Switched to " + test.getName());
     testEditor.setActiveTest(test);
   },
-
   runTester: function() {
     var input = {value:
       app.connector.template.properties["defaultTaskTestsOrder"]};
@@ -574,18 +538,6 @@ var taskPane = {
     }
   },
 
-  refreshTaskPane: function () {
-    if (app.connector == null) {
-      logger.debug("refreshTaskPane: FATAL, connector is null");
-      return;
-    }
-    taskPane.refreshTaskTests();
-    taskPane.refreshTaskMenu();
-    taskPane.refreshArguments();
-    taskPane.refreshData();
-    taskPane.stepTree.refresh();
-  },
-
   /*
    * All refresh* methods are supposed to be simple, redrawing methods called
    * at the very last rendering phase after all business logic has been
@@ -595,12 +547,21 @@ var taskPane = {
    * Those methods are full of unnessary asserts (connector != null,
    * selectedTask != null) handled higher in the chain.
    */
-
+  refreshTaskPane: function () {
+    if (app.connector == null) {
+      logger.debug("refreshTaskPane: FATAL, connector is null");
+      return;
+    }
+    taskPane.refreshTaskTests();
+    taskPane.refreshTaskMenu();
+    taskPane.refreshArguments();
+    taskPane.dataBrowser.refresh();
+    taskPane.stepTree.refresh();
+  },
   refreshTaskMenu: function () {
     var menu = taskPane.taskMenu;
     var popup = menu.getElementsByTagName("menupopup")[0];
     xmlHelper.emptyChildren(popup);
-
     var theSelectedItem = null;
     var defaultItem = null;
     var tasks = app.connector.getTasks();
@@ -620,7 +581,6 @@ var taskPane = {
       if (app.selectedTask && (tasks[i].id == app.selectedTask.id))
         theSelectedItem = item;
     }
-
     if (theSelectedItem) {
       menu.selectedItem = theSelectedItem;
     } else if (defaultItem) {
@@ -631,7 +591,6 @@ var taskPane = {
       menu.selectedIndex = -1;
     }
   },
-
   refreshArguments: function () {
     var list = taskPane.tpDoc.getElementById("cfArgumentList");
     let test = testEditor.getActiveTest();
@@ -678,105 +637,6 @@ var taskPane = {
       }
     }
   },
-
-  refreshData: function () {
-    let task = app.selectedTask;
-    let keys = Task.dataKeys.concat(Task.connectorDataKeys);
-    let tabBox = taskPane.tpDoc.getElementById("cfDataBrowser");
-    xmlHelper.emptyChildren(tabBox);
-
-    let tabs = xmlHelper.appendNode(tabBox, "tabs");
-    for (let i = 0; i < keys.length; i++) {
-      let tab = xmlHelper.appendNode(tabs, "tab", null, {label: keys[i]});
-    }
-
-    let panels = xmlHelper.appendNode(tabBox, "tabpanels", null, { flex: 1, style: 'padding: 0; margin: 0;'  });
-    for (let i = 0; i < keys.length; i++) {
-      let panel = xmlHelper.appendNode(panels, "tabpanel", null, { flex: 1, style: 'padding: 0; margin: 0;' });
-
-      let tree = xmlHelper.appendNode(panel, "tree", null, {
-        flex: 1,
-        value: keys[i],
-        hidecolumnpicker: 'true',
-        style: 'padding: 0; margin: 0;'
-      });
-      // couldn't get the context attribute to work, seems like all this is still necessary
-      tree.addEventListener("contextmenu", taskPane.onDataTreeClick, false);
-
-      let treecols = xmlHelper.appendNode(tree, "treecols");
-      xmlHelper.appendNode(treecols, "treecol", null, {ordinal: 1, label: "Name", flex: 1, primary: true});
-      xmlHelper.appendNode(treecols, "splitter", null, {ordinal: 2, class: "tree-splitter"});
-      xmlHelper.appendNode(treecols, "treecol", null, {ordinal: 3, label: "Value", flex: 3});
-
-      let treechildren = xmlHelper.appendNode(tree, "treechildren", null, { flex: 1 });
-      taskPane.addDataNodes(treechildren, task.data[keys[i]], "");
-    }
-    tabBox.selectedIndex = taskPane.dataTab;
-  },
-
-  addDataNodes: function (root, obj, parentPath, parentName) {
-    for (key in obj) {
-      // TODO: generate JSONpath here
-      var jsPath = parentPath + "/" + key;
-      var item = xmlHelper.appendNode(root, "treeitem", null, {"assertpath": jsPath});
-      var row = xmlHelper.appendNode(item, "treerow");
-      var dispkey = key;
-      if ( parentName && ! parentName.match( /^[0-9]+/ ) )
-        dispkey = parentName + " [" + key + "]";
-      if ( typeof(obj[key]) == "object" && obj[key].length > 1 ) {
-        dispkey = key + " (" + obj[key].length + ")";
-      }
-      //xmlHelper.appendNode(row, "treecell", null, {"label": key });
-      xmlHelper.appendNode(row, "treecell", null, {"label": dispkey });
-      if (typeof obj[key] != "object") {
-        xmlHelper.appendNode(row, "treecell", null, {"label": obj[key]});
-        item.setAttribute("tooltiptext", obj[key]);
-      } else if ((obj[key].length == 1) && (typeof obj[key][0] != "object")) {
-        xmlHelper.appendNode(row, "treecell", null, {"label": obj[key][0]});
-        item.setAttribute("tooltiptext", obj[key][0]);
-      } else {
-        item.setAttribute("container", "true");
-        item.setAttribute("open", "true");
-        var crib = xmlHelper.appendNode(item, "treechildren");
-        if (obj[key].length == 1) {
-          taskPane.addDataNodes(crib, obj[key][0], jsPath, "");
-        } else {
-          taskPane.addDataNodes(crib, obj[key], jsPath, key);
-        }
-      }
-    }
-  },
-
-  onDataTreeClick:  function (evnt) {
-    var tree =  evnt.currentTarget;
-    var treeBox = tree.treeBoxObject;
-    var row = {}, col = {}, obj = {};
-    // row.value has the row index
-    // col.value has the column object
-    treeBox.getCellAt(evnt.clientX, evnt.clientY, row, col, obj);
-    // row out of bound
-    if (row.value == -1) return;
-    // go only for the 'value' column
-    //if (col.value.element.getAttribute("label") == "Name") return;
-    var treeItem = tree.contentView.getItemAtIndex(row.value);
-    var assertPath = treeItem.getAttribute("assertpath");
-    testEditor.lastAssertPath = assertPath;
-    taskPane.tpDoc.getElementById("dataBrowserContextMenu")
-      .openPopup(treeItem, "after_pointer", evnt.clientX, evnt.clientY,
-          true);
-    logger.debug("Tree clicked at address " + assertPath + ". ("
-        +evnt.clientX+","+evnt.clientY+")");
-  },
-
-  copyData: function(item) {
-    let cell = item.parentNode.anchorNode.children[0].children[1];
-    if (typeof cell !== "undefined") {
-      const gClipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].
-      getService(Components.interfaces.nsIClipboardHelper);
-      gClipboardHelper.copyString(cell.getAttribute("label"));
-    }
-  },
-
   refreshTaskTests: function () {
     var menuList = taskPane.tpDoc.getElementById("cfTaskTests");
     var popup = menuList.getElementsByTagName("menupopup")[0];
@@ -792,12 +652,10 @@ var taskPane = {
         }
       }
   },
-
   // To be called every time anything is changed
   enableSave: function () {
     taskPane.tpDoc.getElementById("cfSaveButton").disabled = false;
   },
-
   disableSave: function () {
     taskPane.tpDoc.getElementById("cfSaveButton").disabled = true;
   }
