@@ -53,10 +53,23 @@ var jsonPathHelper = {
   var flagparts = flag.split("-");
   if (flagparts[0] == flagprefix &&
       flagparts[1] == spec.key &&
-      spec.path.substring(0,path.length) == path )
+      spec.path.substring(0,path.length) == path ) {
+    if (0) logger.debug("jsonph: capdef:" +
+      " '" + flagparts[0] + "' == '" + flagprefix + "'. " +
+      " '" + flagparts[1] + "' == '" + spec.key + "'. " +
+      " '" + spec.path.substring(0,path.length) + "' == '" + path + ". " +
+      " Returning TRUE" );
     return true;
+  }
+  if (0) logger.debug("jsonph: capdef:" +
+    " '" + flagparts[0] + "' vs '" + flagprefix + "'. " +
+    " '" + flagparts[1] + "' vs '" + spec.key + "'. " +
+    " '" + spec.path.substring(0,path.length) + "' vs '" + path + ". " +
+    " Returning null" );
+
   return null;
   },
+  
   // This extends the above to apply to arrays of conditions. 
   conditionsCapable: function (conditions, flag, path, flagprefix) {
     if (Array.isArray(conditions)) {
@@ -278,8 +291,8 @@ var jsonPathHelper = {
       if (!src.key) srcObj[dst.key] = srcObjs[i];
       else if (src.key === '*') srcObj = srcObjs[i];
       else srcObj[dst.key] = srcObjs[i][src.key];
-      for (dstKey in srcObj) {
-        srcArr = srcObj[dstKey];
+      for (let dstKey in srcObj) {
+        let srcArr = srcObj[dstKey];
         if ((typeof srcArr !== 'undefined') && (!Array.isArray(srcArr))) srcArr = [srcArr];
         let transformed = func(srcArr);
         if (transformed !== "##NOOP##")
