@@ -177,10 +177,14 @@ var xmlHelper = {
     },
 
     // Create a whole new xml document from a string
-    docFromString: function (str) {
+    // contentType defaults to "text/xml" but can also be "text/html", or 
+    // anything else Mozillas parser handles.
+    docFromString: function (str, contentType) {
+      if (!contentType)
+        contentType = "text/xml";
       var parser = Cc["@mozilla.org/xmlextras/domparser;1"]
         .createInstance(Ci.nsIDOMParser);
-      var doc = parser.parseFromString(str, "text/xml" );
+      var doc = parser.parseFromString(str, contentType );
       return doc.documentElement;
     },
 
