@@ -130,7 +130,8 @@ add_page (GtkNotebook *notebook)
     g_signal_connect(view, "status-changed", G_CALLBACK(status_cb), NULL);
     g_signal_connect(close_button, "clicked", G_CALLBACK(close_cb), view);
 
-    moz_web_view_load_uri(MOZ_WEB_VIEW(view), "chrome://cfengine/content/engine.xul");
+    moz_web_view_load_data(MOZ_WEB_VIEW(view), "file:///page.html",
+                           "text/html; charset=utf-8", "", 0);
     int page = gtk_notebook_append_page(notebook, view, box);
     gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page);
 
