@@ -969,22 +969,27 @@ void yf::CF::Rep::task_connector(Package &package, Z_HTTP_Request *req,
 
 void yf::CF::Rep::release_zsession(Package &package)
 {
-    boost::mutex::scoped_lock lock(m_mutex);
-    std::map<mp::Session,ZSessionPtr>::iterator it;
-
-    it = m_clients.find(package.session());
-    if (it != m_clients.end())
+    ZSessionPtr r_session;
     {
-        if (package.session().is_closed())
+        boost::mutex::scoped_lock lock(m_mutex);
+        std::map<mp::Session,ZSessionPtr>::iterator it;
+
+        it = m_clients.find(package.session());
+        if (it != m_clients.end())
         {
-            m_clients.erase(it);
+            if (package.session().is_closed())
+            {
+                r_session = it->second;
+                m_clients.erase(it);
+            }
+            else
+            {
+                it->second->in_use = false;
+            }
+            m_cond_session_ready.notify_all();
         }
-        else
-        {
-            it->second->in_use = false;
-        }
-        m_cond_session_ready.notify_all();
     }
+    // r_session goes out of scope and ZSession may be destructed
 }
 
 yf::CF::ZSession *yf::CF::Rep::get_zsession(Package &package)
